#version 110
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D DiffuseSampler;
uniform sampler2D DataSampler;
uniform sampler2D DepthSampler;
uniform sampler2D NoiseSampler;
uniform sampler2D BGSampler;

varying vec2 texCoord;
varying vec2 oneTexel;

uniform vec2 InSize;

uniform vec3 Gray;

#define Px 1.0/InSize.x
#define Py 1.0/InSize.y

#define FOV 90.

#define DATA texture2D(DataSampler, vec2(0.0, 0.0)).r
#define FADE texture2D(DataSampler, vec2(Px * 2.0, 0.0)).r
#define OLDI texture2D(DataSampler, vec2(Px * 3.0, 0.0)).r
#define DEPTH texture2D(DepthSampler, effectiveCoord).r
#define TIME toInt(texture2D(DataSampler, vec2(Px * 4.0, 0.0)).rgb)
#define EFFECT_TIME toInt(texture2D(DataSampler, vec2(Px * 5.0, 0.0)).rgb)
#define ToS_ANIM toInt(texture2D(DataSampler, vec2(Px * 6.0, 0.0)).rgb)

vec2 effectiveCoord;
vec4 base;

int toInt(vec3 a) {
    return int(a.r * 255.) + (int(a.g * 255.) +  int(a.b * 255.) * 256) * 256;
}

vec3 toVec3(int a) {
    return vec3(float(a % 256) / 255., float((a % 65536) / 256) / 255., float((a % 16777216) / 65536) / 255.);
}

float rand(vec2 co){
  return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

vec2 hash(vec2 p) {
	p = vec2(dot(p,vec2(127.1,311.7)),
			 dot(p,vec2(269.5,183.3)));
	return -1. + 2.*fract(sin(p+20.)*53758.5453123);
}

float noise(vec2 p) {
    vec2 i = floor(p), f = fract(p);
	vec2 u = f*f*(3.-2.*f);
    return mix(mix(dot(hash(i + vec2(0.,0.)), f - vec2(0.,0.)), 
                   dot(hash(i + vec2(1.,0.)), f - vec2(1.,0.)), u.x),
               mix(dot(hash(i + vec2(0.,1.)), f - vec2(0.,1.)), 
                   dot(hash(i + vec2(1.,1.)), f - vec2(1.,1.)), u.x), u.y);
}

float distanceFromDepth(float d) {
    return length(vec3(1., (2. * effectiveCoord - 1.) * vec2(InSize.x / InSize.y, 1.) * tan(radians(FOV / 2.))) * d);
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

void heat(float a) {
    vec2 temp = effectiveCoord;
    temp.y -= float(TIME) * 0.001;

    vec4 dst_map_val = texture2D(NoiseSampler, temp);

    vec2 dst_offset = dst_map_val.xy;
    dst_offset -= vec2(0.5,0.5);
    dst_offset *= 2.0;
    dst_offset *= 0.01;

    float depth = DEPTH;
    float depth2 = depth - 0.1 < 0.0 ? 0.0 : depth - 0.1;

    vec2 dist_tex_coord = effectiveCoord + dst_offset * depth2;
    float depth3 = texture2D(DepthSampler, dist_tex_coord).r;

    dist_tex_coord = mix(effectiveCoord, dist_tex_coord, a);
    
    if(depth3 > 0.1) effectiveCoord = dist_tex_coord;
}

void heatDistort(bool current) {
    int time = TIME - EFFECT_TIME;
    float a = float(time) / 75.0;
    if(a > 1.0) a = 1.0;
    if(!current) a = 1.0 - a;

    heat(a);    
}

vec4 conBriSat(vec4 temp, float c, float b, float s) {
    float doesthecompilerstilllikethis = 0.5;

    temp.rgb = ((temp.rgb - doesthecompilerstilllikethis) * c) + doesthecompilerstilllikethis + b;

    float Luma = dot(temp.rgb, Gray);
    vec3 Chroma = temp.rgb - Luma;
    temp.rgb = (Chroma * c * s) + Luma;
    return temp;
}

vec4 rgbControl(vec4 temp, float r, float g, float b) {
    temp.rgb *= vec3(r, g, b);
    return temp;
}

vec4 vn(vec4 temp) {
    float currBright = sqrt(pow(temp.r, 2.0) + pow(temp.g, 2.0) + pow(temp.b, 2.0));

    temp.rgb *= abs(currBright - 0.1) / 2.0;

    if (currBright < 0.3) {
        //OutColor[0] *= 0.0;
        //OutColor[1] *= 0.0;
        temp.b *= 1.05;
    } else if (currBright > 0.7) {
        temp.r += 0.1;
        temp.g += 0.065;
        //OutColor[2] *= 0.0;
    }

    return temp;
}

vec4 sand(vec4 temp) {
    float time = float(TIME);
    float a = noise(effectiveCoord * InSize / 30.) * sin(time / 20.) + noise(effectiveCoord * InSize / 1. - time) + abs(noise(effectiveCoord * InSize / 100. + time / 3.));
    if(a < 0.) a = 0.;

    vec2 center = vec2(pow(0.5 - effectiveCoord.x, 2.), pow(0.5 - effectiveCoord.y, 2.));
    float dist = center.x + center.y;
    a *= dist;
    
    //return temp + vec4(a * 0.88, a * 0.78, a * 0.5, 0.0);
    return mix(temp, vec4(0.88, 0.78, 0.5, 1.0), a);
}

vec4 fog(vec4 temp, float d, float o, vec4 base) {
    float depth = distanceFromDepth(DEPTH) * d;

    float fogAmount = clamp(depth, 0., 1.);

    return mix(temp, base, fogAmount * o);
}

vec4 channelOffset(float d, float a) {
    float s = sin(a);
    float c = cos(a);

    vec2 rc = effectiveCoord;
    float r = texture2D(DiffuseSampler, rc).r;
   
    vec2 gc = effectiveCoord + vec2(s * d, c * d);
    float g = texture2D(DiffuseSampler, gc).g;

    vec2 bc = effectiveCoord - vec2(s * d, c * d);
    float b = texture2D(DiffuseSampler, bc).b;

    return vec4(r, g, b, 1.0);
}

float vignette() {
    vec2 center = vec2(pow(0.5 - effectiveCoord.x, 2.), pow(0.5 - effectiveCoord.y, 2.));
    return center.x + center.y;
}

vec4 warped() {
    int time = TIME;
    float time_s = float(TIME) / 75.0;

    float vignette = vignette();

    float d = 0.010;
    d += time % 7500 <  75 ? sin(float(time % 7500) / 47.7464829) * 0.030 : 0.0;
    d -= time % 750  <  15 ? sin(float(time %  750) / 47.7464829) * 0.100 : 0.0;
    d += time %  75  <  15 ? sin(float(time %   75) / 4.77464829) * 0.001 : 0.0;
    d *= vignette * 2.0;

    return channelOffset(d, time_s);
}

vec2 distort(float d) {
    vec2 pos = effectiveCoord - vec2(0.5, 0.5);
    pos *= vec2(pow(length(pos), d));
    pos += vec2(0.5, 0.5);
    return pos;
}

void retroDistort(bool current) {
    int time = TIME - EFFECT_TIME;
    float a = float(time) / 75.0 * 0.05;
    if(a > 0.05) a = 0.05;
    if(!current) a = 0.05 - a;
    effectiveCoord = distort(a);
}

vec4 retro(vec4 temp) {
    int r, g, b; int f = 32;
    r = int(temp.r * 255.0); g = int(temp.g * 255.0); b = int(temp.b * 255.0);
    r /= f; g /= f; b /= f; r *= f; g *= f; b *= f;
    temp.rgb = vec3(float(r) / 255.0, float(g) / 255.0, float(b) / 255.0); 
    return temp;
}

vec4 crt(vec4 temp) {
    float time = float(TIME) / 75.0;

    temp.rgb += abs(sin(effectiveCoord.y * 100.0 + time * 5.0)) * 0.016;
    temp.rgb += abs(sin(effectiveCoord.y * 300.0 - time * 10.0)) * 0.01;

    return temp;
}

vec4 pirate(vec4 temp) {
    int time = TIME;
    float time_s = float(TIME) / 75.0;
    float depth = distanceFromDepth(DEPTH);

    vec4 dust = depth * clamp(noise(effectiveCoord * InSize - time_s), 0.0, 1.0) * clamp(noise(effectiveCoord * vec2(InSize.x, InSize.y / 10.0) / 30.0 - time_s), 0.0, 1.0)  * clamp(noise(effectiveCoord * InSize / 30.0 - time_s), 0.0, 1.0) * vec4(0.0, 1.0, 0.0, 1.0) * 10.;

    return temp + dust;
}

vec4 white_noise(vec4 temp, float a) {
    vec2 pos = effectiveCoord;
    pos *= sin(float(TIME) / 75.0);
    float r = rand(pos);
    vec4 noise = vec4(r);
    return mix(temp, noise, a);
}

void hallowDistort(bool current) {
    int time = TIME - EFFECT_TIME;
    float a = float(time) / 18.75 * -0.05;
    if(a < -0.05) a = -0.05;
    if(!current) a = -0.05 - a;
    effectiveCoord = distort(a);
}

vec4 hallowForest(vec4 temp) {
    int time = TIME;
    float vignette = vignette();
    int v = int(vignette * 255.0);

    float hue = float((time + v * 2) % 256) / 255.0;
    vec3 color = mix(temp.rgb, hsv2rgb(vec3(hue, 1.0, 1.0)), vignette * 4.0);
    color = color * 0.2 + 0.8;

    return rgbControl(temp, color.r, color.g, color.b);
}

vec2 rotate(float a) {
    vec2 U = vec2(cos(a),sin(a));
    vec2 V = vec2(-U.y,+U.x);
    vec2 O = vec2(0.5,0.5); 
    float u = dot(effectiveCoord-O,U);
    float v = dot(effectiveCoord-O,V);
    return vec2(u + 0.5, v + 0.5);
}

void spin(bool current) {
    int time = TIME - EFFECT_TIME;
    float a = float(time) / 37.5 * 1.57;
    if(a > 3.1415926535) a = 3.1415926535;
    if(!current) a += 3.1415926535;

    effectiveCoord = rotate(a);
}

vec4 bloom(vec4 temp) {
    float pixelWidth =  Px * 2.0;
    float pixelHeight = Py * 2.0;
    float dim = 0.5;
    float b = length(temp.rgb);
	float glow = b * ((pixelWidth + pixelHeight) / 2.0);
	vec4 bloom = vec4(0);

	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x, effectiveCoord.y)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x - glow, effectiveCoord.y - glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x + glow, effectiveCoord.y + glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x + glow, effectiveCoord.y - glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x - glow, effectiveCoord.y + glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x + glow, effectiveCoord.y)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x - glow, effectiveCoord.y)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x, effectiveCoord.y + glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x, effectiveCoord.y - glow)) - dim);
	bloom = clamp(bloom / 3.0, 0.0, 1.0);

    float time = float(TIME)/ 75.0;
	temp += bloom * clamp(abs(sin(time) * sin(time * 2.0)), 0.5, 1000.0);
    return temp;
}

vec4 strats(vec4 temp) {
    vec3 hsv = rgb2hsv(temp.rgb);
    float depth = DEPTH;
    if(depth > 0.497) {
        vec3 rgb = hsv2rgb(vec3(hsv.x, 0.0, hsv.z));
        return vec4(rgb, 1.0);
    }

    if(hsv.x < 0.1 || hsv.x > 0.9) {
        if(hsv.y > 0.45) return temp;
        else {
            vec3 rgb = hsv2rgb(vec3(hsv.x, 0.0, hsv.z));
            return vec4(rgb, 1.0);
        }
    } else {
        if(hsv.y > 0.6) return temp;
        else {
            vec3 rgb = hsv2rgb(vec3(hsv.x, 0.0, hsv.z));
            return vec4(rgb, 1.0);
        }
    }
}

vec4 piratefog(vec4 temp) {
    float depth = distanceFromDepth(DEPTH) * 10.0;
    vec3 hsv = rgb2hsv(temp.rgb);

    float fogAmount = clamp(depth, 0.0, 1.0);
    float greenness = clamp((hsv.z - 0.5) * 1.0, 0.0, 1.0) * fogAmount;
    fogAmount *= clamp(1.5 - hsv.z, 0.0, 1.0);

    temp.rb *= 1.0 - greenness;
    temp.g  *= 1.0 + greenness;

    return mix(temp, vec4(0.0, 0.2, 0.2, 1.0), fogAmount * 0.95);
}

vec4 outline() {
    vec4 center = base;
    if(length(center) > 1.2) return center;
    vec4 up    = texture2D(DiffuseSampler, effectiveCoord + vec2(        0.0, -oneTexel.y));
    vec4 down  = texture2D(DiffuseSampler, effectiveCoord + vec2( oneTexel.x,         0.0));
    vec4 left  = texture2D(DiffuseSampler, effectiveCoord + vec2(-oneTexel.x,         0.0));
    vec4 right = texture2D(DiffuseSampler, effectiveCoord + vec2(        0.0,  oneTexel.y));
    vec4 uDiff = center - up;
    vec4 dDiff = center - down;
    vec4 lDiff = center - left;
    vec4 rDiff = center - right;
    vec4 sum = uDiff + dDiff + lDiff + rDiff;

    float depth = DEPTH;

    vec3 clamped;
    if(length(sum) > 0.9 && depth < 0.497) {
        clamped = clamp(center.rgb + sum.rgb, 0.0, 1.0);
    } else {
        clamped = clamp(center.rgb * 0.4 + sum.rgb, 0.0, 1.0);
    }
    return vec4(clamped, 1.0);
}

vec4 crimson(vec4 temp) {
    vec3 hsv = rgb2hsv(temp.rgb);

    float depth = DEPTH;
    if(depth > 0.497) {
        float f = .8 - temp.r;
        if(f < 0.) f = 0.;
        return vec4(hsv2rgb(vec3(hsv.x , hsv.y, hsv.z - 0.3)), 1.0) + rand(effectiveCoord) * noise(effectiveCoord * InSize / 1. + float(TIME) * 0.01) * f;
    }

    if(abs(hsv.x - 0.666) < 0.166) {
        return vec4(hsv2rgb(vec3(hsv.x, hsv.y * 1.5, hsv.z)), 1.0);
    } else if(abs(hsv.x - 0.333) < 0.166) {
        return vec4(hsv2rgb(vec3(hsv.x, hsv.y * 1.5, hsv.z)), 1.0);
    } else if(hsv.x < 0.5) {
        return vec4(hsv2rgb(vec3(hsv.x * 0.333, hsv.y, hsv.z)), 1.0);
    } else {
        return vec4(hsv2rgb(vec3((2.0 + hsv.x) / 3.0, hsv.y, hsv.z)), 1.0);
    }
}

vec4 basalt(vec4 temp) {
    if(length(temp.rgb) < 0.02) temp.b += 0.05;
    return temp;
}

vec4 monu(vec4 temp) {
    float depth = DEPTH;

    vec3 hsv = rgb2hsv(temp.rgb);
    temp.rgb = hsv2rgb(vec3(hsv.x, hsv.y + depth / 5.0, hsv.z));
    return temp;
}

vec4 bloom2(vec4 temp) {
    float pixelWidth =  Px * 2.0;
    float pixelHeight = Py * 2.0;
    float dim = 0.5;
    float b = length(temp.rgb);
	float glow = b * ((pixelWidth + pixelHeight) / 2.0);
	vec4 bloom = vec4(0);

	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x, effectiveCoord.y)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x - glow, effectiveCoord.y - glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x + glow, effectiveCoord.y + glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x + glow, effectiveCoord.y - glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x - glow, effectiveCoord.y + glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x + glow, effectiveCoord.y)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x - glow, effectiveCoord.y)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x, effectiveCoord.y + glow)) - dim);
	bloom += (texture2D(DiffuseSampler, vec2(effectiveCoord.x, effectiveCoord.y - glow)) - dim);
	bloom = clamp(bloom / 4.0, 0.0, 1.0);

	temp += bloom;
    return temp;
}

/*
vec4 startCheck() {
    float time = float(TIME) / 75.0 / 3.0;
    vec3 a = rgb2hsv(texture2D(BGSampler, vec2(texCoord.x, -texCoord.y)).rgb);
    vec4 b = vec4(hsv2rgb(vec3(time, 1.0, pow(abs(texCoord.x - 0.5), 2.0))), 1.0);
    return vec4(hsv2rgb(vec3(time - texCoord.x - texCoord.y, a.g, a.b)), 1.0) + b;
}
*/

vec4 basaltBase(vec4 temp) {
    float a = length(temp.rgb);
    if(a < 0.25) {
        a = clamp(1.0 - (a - 0.2) * 20.0, 0.0, 1.0);
        temp.rgb *= 1.0 + a * 0.8;
    } 
    return temp;
}

void setDistorsion(int id, bool current) {
        if(id == 36) heatDistort(current);
        else if(id == 38) heatDistort(current);
        else if(id == 40) retroDistort(current);
        else if(id == 41) heatDistort(current);
        else if(id == 43) hallowDistort(current);
        else if(id == 44) spin(current);
        else if(id == 49) heatDistort(current);
        else if(id == 51) heatDistort(current);
        else if(id == 52) heatDistort(current);
}

//I'd use switch cases for these, but intel decided it hates me and those don't exist on their cards.

vec4 getEffect(int id, bool current) {
        if(id == 36) return base; //Normal heat
        if(id == 37) return vn(rgbControl(conBriSat(base, 1.1, 0.1, 0.4), 0.9, 0.8, 1.0)); //Vacuum nightmares
        if(id == 38) return fog(rgbControl(sand(base), 0.8, 0.8, 0.6), 10.0, 0.5, vec4(0.88, 0.58, 0.4, 1.0)); //Sandstorm
        if(id == 39) return white_noise(warped(), 0.05); //Warped
        if(id == 40) return crt(retro(base)); //Retro
        if(id == 41) return conBriSat(piratefog(rgbControl(base, 1.6, 1.2, 1.0)), 1.4, 0.1, 0.5); //Un un-ojado pirata
        if(id == 42) return conBriSat(rgbControl(base, 1.0, 0.5, 0.0), 1.4, 0.1, 0.5); //Unused
        if(id == 43) return hallowForest(conBriSat(bloom(channelOffset(0.001, 1.57)), 0.7, -0.2, 1.5)); //Hallow 2
        if(id == 44) return base; //It do be spinning
        if(id == 45) return white_noise(conBriSat(crimson(base), 1.05, 0.0, 1.0), 0.04); //Crimson
        if(id == 46) return conBriSat(bloom(base), 0.7, -0.2, 1.2); //Hallow
        if(id == 47) return strats(base); //Start
        if(id == 48) return rgbControl(outline(), 1.0, 0.9, 0.7); //Meteorites
        if(id == 49) return rgbControl(base, 1.0, 0.8, 0.5); //Lava sea
        if(id == 50) return basalt(fog(conBriSat(basaltBase(base), 1.3, 0.0, 0.7), 2.0, 1.0, vec4(0.1, 0.0, 0.0, 1.0))); //Basalt deltas
        if(id == 51) return conBriSat(monu(base), 1.1, 0.0, 1.0); //Monument
        if(id == 52) return conBriSat(bloom2(base), 1.05, -0.05, 0.8); //Temple of sun
        if(id == 53) return fog(rgbControl(base, 0.8, 0.8, 0.6), 10.0, 0.5, vec4(0.88, 0.58, 0.4, 1.0));
        return base; //Normal
}

void main() {

    int data = int(255.0 * DATA);
    float fade = FADE;
    effectiveCoord = texCoord;

    if(fade != 1.0) {
        int old = int(255.0 * OLDI);
        setDistorsion(data, true);
        setDistorsion(old, false);
        base = texture2D(DiffuseSampler, effectiveCoord);
        vec4 v_new = getEffect(data, true);
        vec4 v_old = getEffect(old, false);
        gl_FragColor = mix(v_old, v_new, fade);
    } else {
        setDistorsion(data, true);
        base = texture2D(DiffuseSampler, effectiveCoord);
        float tos = float(ToS_ANIM) / 75.0;
        if(tos > 0.0) {
            float tos2 = 1.0 - tos; if(tos2 < 0.0) tos2 = 0.0;
            base = channelOffset((sin(tos) * 0.01 + sin(tos * 5.0) * sin(tos * 10.0) * 0.01) * tos / 5.0 * (vignette() + 0.05) * 3.0, sin(tos));
        }
        gl_FragColor = getEffect(data, true);
    }

    //vec4 debug = texture2D(DataSampler, texCoord / 100.);
    //if(debug != vec4(0.0, 0.0, 0.0, 0.0)) gl_FragColor = debug;
    if(texCoord.x < Px && texCoord.y < Py) gl_FragColor = vec4(1.0,1.0,1.0,1.0);
    
}