#version 110
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D DiffuseSampler;
uniform sampler2D DataSampler;
uniform sampler2D ImageSampler;

varying vec2 texCoord;
varying vec2 oneTexel;

uniform vec2 InSize;

#define Px 1.0/InSize.x
#define Py 1.0/InSize.y

#define DATA texture2D(DataSampler, vec2(0.0, 0.0)).r
#define FADE texture2D(DataSampler, vec2(Px * 2.0, 0.0)).r
#define OLDI texture2D(DataSampler, vec2(Px * 3.0, 0.0)).r

vec4 test1() {
    vec4 temp = texture2D(DiffuseSampler, texCoord);
    temp.r *= 0.0;
    return temp;
}

vec4 test2() {
    vec4 temp = texture2D(DiffuseSampler, texCoord);
    temp.g *= 0.0;
    return temp;
}

vec4 test3() {
    vec4 temp = texture2D(DiffuseSampler, texCoord);
    temp.b *= 0.0;
    return temp;
}

vec4 test4() {
    vec4 temp = texture2D(DiffuseSampler, vec2(texCoord.x, 1.0-texCoord.y));
    return temp;
}

vec4 getEffect(int id) {
    switch(id) {
        case 27: return test1();
        case 28: return test2();
        case 29: return test3();
        case 30: return test4();
        default: return texture2D(DiffuseSampler, texCoord);
    }
}

void main() {
    
    int data = int(255.0 * DATA);
    float fade = FADE;

    if(fade != 1.0) {
        int old = int(255.0 * OLDI);
        vec4 v_new = getEffect(data);
        vec4 v_old = getEffect(old);
        gl_FragColor = mix(v_old, v_new, fade);
    } else gl_FragColor = getEffect(data);

    if(texCoord.y < Py * 100.0) {
        if(texCoord.x < Px * 100.0) gl_FragColor += texture2D(ImageSampler, vec2(float(data / 100) * 0.1 + texCoord.x, -texCoord.y * 10.0));
        else if(texCoord.x < Px * 200.0) gl_FragColor += texture2D(ImageSampler, vec2(float((data % 100) / 10) * 0.1 + texCoord.x - Px * 100.0, -texCoord.y * 10.0));
        else if(texCoord.x < Px * 300.0) gl_FragColor += texture2D(ImageSampler, vec2(float(data % 10) * 0.1 + texCoord.x - Px * 200.0, -texCoord.y * 10.0));
    }
    
}